from mower.solvers.navigation import NavigationSolver
from mower.utils import config
from mower.utils.graph.utils import SceneGraphSolver
from mower.utils.log import logger
from mower.utils.scene import Scene

from .auto_fight import AutoFight
from .battle_choose import BattleChooseSolver
from .battle_choose.choose_support import ChooseSupportSolver


class FightSolver(SceneGraphSolver):
    def run(
        self,
        auto_choose: bool = True,
        last_lack: list = [],
    ):
        work_conf = config.conf.work.works[0]
        work_data = config.works[work_conf["stage"]]
        opers = [oper.model_dump(exclude_none=True) for oper in work_data.opers]
        groups = [group.model_dump(exclude_none=True) for group in work_data.groups]
        actions = [action.model_dump(exclude_none=True) for action in work_data.actions]
        self.level_name = work_conf["stage"]
        support_oper = work_conf["support_oper"]
        self.no_potion = False
        success = False

        self.check_opers_skill(opers, groups)
        if support_oper:
            support_oper = self.find_support_oper(support_oper, opers, groups)
        if auto_choose:
            super().run()
            if self.no_potion:
                return success
            battle_choose_solver = BattleChooseSolver()
            flag = battle_choose_solver.run(
                self.level_name,
                opers,
                groups,
                support_oper,
            )
            self.lack = battle_choose_solver.lack
            if not flag:
                return success
        elif last_lack:
            super().run()
            if self.no_potion:
                return success
            self.lack = last_lack
            ChooseSupportSolver().run(last_lack)

        while work_conf["retry_times"] + 1 > 0:
            super().run()
            if self.no_potion:
                break
            if AutoFight().run(self.level_name, opers, actions, groups):
                success = True
                config.conf.work.works.pop(0)
                break
            work_conf["retry_times"] -= 1
        config.save_conf()
        return success

    def check_opers_skill(self, opers: list, groups: list):
        for oper in opers:
            if "skill" not in oper:
                oper["skill"] = 1
            if "skill_usage" not in oper:
                oper["skill_usage"] = 0
        for group in groups:
            for oper in group["opers"]:
                if "skill" not in oper:
                    oper["skill"] = 1
                if "skill_usage" not in oper:
                    oper["skill_usage"] = 0

    def find_support_oper(self, support_oper: str, opers: list, groups: list):
        for op in opers + groups:
            if op["name"] == support_oper:
                op["support"] = True
                return [op]

    def transition(self):
        if (scene := self.scene()) == Scene.OPERATOR_RECOVER_POTION:
            if config.conf.work.use_medicine:
                logger.info("体力不足，使用理智药")
                self.tap((1635, 865))
                return
            else:
                logger.info("体力不足，未设置使用理智药")
                self.no_potion = True
                return True
        elif scene == Scene.OPERATOR_RECOVER_ORIGINITE:
            logger.info("体力不足，没有理智药")
            self.no_potion = True
            return True
        elif scene == Scene.OPERATOR_SELECT:
            return True
        elif scene == Scene.OPERATOR_BEFORE:
            self.tap_element("ope_start", interval=2)
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            NavigationSolver().run(self.level_name, mode="copy")
