import json

from mower.solvers.fight import FightSolver
from mower.utils import config
from mower.utils.log import logger
from mower.utils.path import get_path


class Oper:
    def __init__(self, name, skill=1):
        self.name = name
        self.skill = skill

    # 定义哈希函数，以便在 set 中去重
    def __hash__(self):
        return hash((self.name, self.skill))

    # 定义相等性判断
    def __eq__(self, other):
        if isinstance(other, Oper):
            return self.name == other.name and self.skill == other.skill
        return False


class CopyWorksSolver:
    def run(self):
        # if config.conf.work.try_select_all:
        #    self.init_all_opers_and_groups()
        self.last_stage = ""
        self.last_lack = []
        while works := config.conf.work.works:
            self.auto_choose = True

            for i in range(len(works)):
                logger.info(
                    f"自动战斗列表：({i + 1}/{len(works)}) 关卡：{works[i]["stage"]} 剩余重试次数：{works[i]['retry_times']} 指定助战：{works[i]['support_oper']}"
                )
            if (
                works[0]["stage"].endswith("#")
                and works[0]["stage"][:-1] == self.last_stage
            ):
                self.auto_choose = False

            self.last_stage = works[0]["stage"]  # 上一次关卡
            fight_solver = FightSolver()
            success = fight_solver.run(self.auto_choose, self.last_lack)
            self.last_lack = fight_solver.lack  # 上一次缺的干员
            if not success:
                return

    def init_all_opers_and_groups(self):
        all_opers = set()
        all_groups = set()
        for work in config.conf.work.works:
            work_path = get_path(f"@app/works/{work["stage"]}.json")
            with work_path.open("r", encoding="utf-8") as f:
                work_data = json.load(f)
                for oper in work_data["opers"]:
                    name = oper["name"]
                    skill = oper.get("skill", 1)
                    all_opers.add(Oper(name, skill))
                for group in work_data["groups"]:
                    all_groups.add(group)
        self.all_groups = [
            group
            for group in all_groups
            if not any(
                Oper(op["name"], op.get("skill", 1)) in all_opers
                for op in group["opers"]
            )
        ]
        self.all_opers = list({"name": op.name, "skill": op.skill} for op in all_opers)
        if len(all_opers) + len(all_groups) <= 12:
            self.auto_choose = False
