import { deepcopy } from '@/utils/deepcopy'
import axios from 'axios'
import { defineStore } from 'pinia'
import { inject, ref, watchEffect } from 'vue'

export const useConfigStore = defineStore('config', () => {
  const shop_list = ref([])

  async function load_shop() {
    const { data } = await axios.get(`${import.meta.env.VITE_HTTP_URL}/shop`)
    const mall_list = []
    for (const i of data) {
      mall_list.push({
        value: i,
        label: i
      })
    }
    shop_list.value = mall_list
  }

  const conf = ref({ webview: { scale: 1.0 } })

  const int2bool = [
    'enable_party',
    'leifeng_mode',
    'maa_enable',
    'maa_rg_enable',
    'mail_enable',
    'skland_enable'
  ]

  const str2list = ['free_blacklist', 'reload_room', 'maa_mall_buy', 'maa_mall_blacklist']

  async function load_config() {
    const { data } = await axios.get(`${import.meta.env.VITE_HTTP_URL}/conf`)

    for (const i of int2bool) {
      data[i] = data[i] != 0
    }
    for (const i of str2list) {
      data[i] = data[i] == '' ? [] : data[i].split(',')
    }
    data.package_type = data.package_type == 1 ? 'official' : 'bilibili'
    data.resting_threshold = data.resting_threshold * 100
    data.tap_to_launch_game.enable = data.tap_to_launch_game.enable ? 'tap' : 'adb'
    conf.value = data
  }

  function build_config() {
    const data = deepcopy(conf.value)

    for (const i of int2bool) {
      data[i] = data[i] ? 1 : 0
    }
    for (const i of str2list) {
      data[i] = data[i].join(',')
    }
    data.package_type = data.package_type == 'official' ? 1 : 0
    data.resting_threshold = data.resting_threshold / 100
    data.tap_to_launch_game.enable = data.tap_to_launch_game.enable == 'tap'

    return data
  }

  const loaded = inject('loaded')
  watchEffect(() => {
    if (loaded.value) {
      axios.post(`${import.meta.env.VITE_HTTP_URL}/conf`, build_config())
    }
  })

  return {
    load_config,
    load_shop,
    shop_list,
    conf
  }
})
